--- Fl_Image.cxx	2007-05-12 19:03:40.000000000 +0200
+++ Fl_Image.cxx	2007-05-13 21:27:14.000000000 +0200
@@ -186,7 +186,8 @@
 		dx, dy,		// Destination coordinates
 		xerr, yerr,	// X & Y errors
 		xmod, ymod,	// X & Y moduli
-		xstep, ystep;	// X & Y step increments
+		xstep, ystep,	// X & Y step increments
+                line_d;
 
 
   // Figure out Bresenheim step/modulus values...
@@ -194,6 +195,7 @@
   xstep  = (w() / W) * d();
   ymod   = h() % H;
   ystep  = h() / H;
+  line_d = ld() ? ld() : w() * d();
 
   // Allocate memory for the new image...
   new_array = new uchar [W * H * d()];
@@ -202,9 +204,7 @@
 
   // Scale the image using a nearest-neighbor algorithm...
   for (dy = H, sy = 0, yerr = H, new_ptr = new_array; dy > 0; dy --) {
-    for (dx = W, xerr = W, old_ptr = array + sy * (w() * d() + ld());
-	 dx > 0;
-	 dx --) {
+    for (dx = W, xerr = W, old_ptr = array + sy * line_d; dx > 0; dx --) {
       for (c = 0; c < d(); c ++) *new_ptr++ = old_ptr[c];
 
       old_ptr += xstep;
@@ -257,22 +257,25 @@
   // Update the image data to do the blend...
   const uchar	*old_ptr;
   int		x, y;
+  int           line_d;
+
+  line_d = (ld() == 0) ? w() * d() : ld();
 
   if (d() < 3) {
     ig = (r * 31 + g * 61 + b * 8) / 100 * (256 - ia);
 
-    for (new_ptr = new_array, old_ptr = array, y = 0; y < h(); y ++, old_ptr += ld())
+    for (new_ptr = new_array, old_ptr = array, y = 0; y < h(); y ++, old_ptr += line_d)
       for (x = 0; x < w(); x ++) {
-	*new_ptr++ = (*old_ptr++ * ia + ig) >> 8;
-	if (d() > 1) *new_ptr++ = *old_ptr++;
+	*new_ptr++ = (old_ptr[x*d()] * ia + ig) >> 8;
+	if (d() > 1) *new_ptr++ = old_ptr[x*d()+1];
       }
   } else {
-    for (new_ptr = new_array, old_ptr = array, y = 0; y < h(); y ++, old_ptr += ld())
+    for (new_ptr = new_array, old_ptr = array, y = 0; y < h(); y ++, old_ptr += line_d)
       for (x = 0; x < w(); x ++) {
-	*new_ptr++ = (*old_ptr++ * ia + ir) >> 8;
-	*new_ptr++ = (*old_ptr++ * ia + ig) >> 8;
-	*new_ptr++ = (*old_ptr++ * ia + ib) >> 8;
-	if (d() > 3) *new_ptr++ = *old_ptr++;
+	*new_ptr++ = (old_ptr[x*d()+0] * ia + ir) >> 8;
+	*new_ptr++ = (old_ptr[x*d()+1] * ia + ig) >> 8;
+	*new_ptr++ = (old_ptr[x*d()+2] * ia + ib) >> 8;
+	if (d() > 3) *new_ptr++ = old_ptr[x*d()+3];
       }
   }
 
@@ -306,11 +309,17 @@
   // Copy the image data, converting to grayscale...
   const uchar	*old_ptr;
   int		x, y;
+  int           line_d;
+
+  line_d = (ld() == 0) ? w() * d() : ld();
+
+  for (new_ptr = new_array, old_ptr = array, y = 0; y < h(); y ++, old_ptr += line_d)
+    for (x = 0; x < w(); x ++) {
+      *new_ptr++ = (uchar)((31 * old_ptr[x*d()+0] +
+                            61 * old_ptr[x*d()+1] +
+                             8 * old_ptr[x*d()+2]) / 100);
 
-  for (new_ptr = new_array, old_ptr = array, y = 0; y < h(); y ++, old_ptr += ld())
-    for (x = 0; x < w(); x ++, old_ptr += d()) {
-      *new_ptr++ = (uchar)((31 * old_ptr[0] + 61 * old_ptr[1] + 8 * old_ptr[2]) / 100);
-      if (d() > 3) *new_ptr++ = old_ptr[3];
+      if (d() > 3) *new_ptr++ = old_ptr[x*d()+3];
     }
 
   // Free the old array as needed, and then set the new pointers/values...
